* ------------------------------------------------------------------*
*                     AVNET DESIGN SERVICES                         *
*                GP32APP.ASM       REV  1.0  10-25-99               *
*                                                                   *
*      *This program is meant for demonstration purposes only*      *
*       ******* NO WARRANTY IS EXPRESSED OR IMPLIED ********        *
*                                                                   *
*                                                                   *
* ------------------------------------------------------------------*
*
*      MC68HC908GP32 Interface
*
* LCD module (1x16 display, 4-bit interface)
*  Connections are made to Port B
*  PB4-PB7 = DB4-DB7
*  PB1 = RS (register select)
*  PB2 = RW (Read ! Write)
*  PB3 = E_LCD (enable)
*
* Temperature sensor
*  Connected to Port B
*  PB0 (AD0)
*
* Piezoelectric Speaker
*  Connected to Port D
*  PD7 (T2CH1)
*
* LED   Connected to Port C
*       PC4 (High Current i/o pin)
*
* -------------------------------------------------------------------


  base 10T                         ;Decimal as default number base
  include "gp32head.asm"           ;Header file defines registers, etc.

* -------------------------------------------------------------------
* CHANGE THIS TO MATCH OPPORATING FREQUENCY (MHz)
* To allow adaptable delays.
* -------------------------------------------------------------------
BUSFREQ  EQU  2
USCOUNT  EQU  $0E
USDELAY  EQU  BUSFREQ*USCOUNT

; Miscellaneous equates
DELAYVAL   EQU  $50   ;Default value for delays
PRESCALER2 EQU          %00000010  ;Prescaler bits determine timer freq.
PRESCALER  EQU          %00000000  ;Prescaler bits determine timer freq.

; ASCII character definitions
SPACE      EQU  $20   ;ASCII space
EOT        EQU  $04   ;ASCII end-of-text
CR         EQU  $0D   ;ASCII carriage return
LF         EQU  $0A   ;ASCII linefeed

TABLEEND EQU  $FF

;LCD Control lines, E = enable, RS = Register Select (Data vs. Instruction)
;R/W = Read ! Write; Refers to Port B pins
RS  EQU  1
RW  EQU          2
E_LCD  EQU  3

MAXCHARS EQU          $10    ;Maximum characters per line of LCD
MAXLINE  EQU  $08

LastMode EQU  $02   ;Number of the last Demo

TOFMAXCOUNT     EQU  2   ;Changed for monitor mode debugging (/2)
;TOFMAXCOUNT    EQU         1


* -------------------------------------------------------------------
* RAM variables
* -------------------------------------------------------------------
 ORG    RAMSPACE    ;Start of RAM
Counter    RMB  1         ;Counter variable for delay
TempVar  RMB  1   ;Temporary 8-bit storage
TempWord RMB  2   ;Temporary 16-bit storage
Mode  RMB  1   ;number of current demo
NextMode  RMB  1   ;number of next demo to run
*-------------------------------------------------------------
Count  RMB          1   ;Counter variable
MsgIndex   RMB          2     ;Index counter variable
MsgStart   RMB          2   ;Stores starting point of string
TOFCount RMB  1
EventCount  RMB  1
TempPtr  RMB  2

* -------------------------------------------------------------------
* Program code
* -------------------------------------------------------------------
 ORG FLASHSPACE ;Start of FLASH memory


Start:

 ;----------------------------------------------------------------
 ;These instructions setup the clock generator to use a 32.768kHz
 ;crystal as a source for the bus clock
 ;
 ;these lines are comented out when using an external Oscillator
 ;----------------------------------------------------------------

  ;BCLR           PLLON,PCTL              ;turn off PLL so it can be initialized
  ;MOV  #$00,PMSH  ;Set multiplier for 4.9152MHz
  ;MOV  #$96,PMSL  ;see manual for calculations
  ;MOV  #$80,PMRS  ;Set range select
  ;BSET         AUTO,PBWC  ;Allow automatic acquisition & tracking
  ;BSET         PLLON,PCTL  ;turn PLL back on
  ;BRCLR         LOCK,PBWC,*  ;Wait for PLL to lock
  ;BSET         BCS,PCTL  ;Select PLL as Source


 ;-------------------------------------------------
 ;908GP32 Initialization
 ;Setup configuration registers
 ;-------------------------------------------------

  MOV  #$0B,CONFIG1  ;Set LVI5OR3 and COPD, enable STOP
  MOV  #$03,CONFIG2  ;Select bus as SCI clock source
                NOP
  NOP
                CLR             PORTA
                CLR             PORTB
                CLR             PORTC
                CLR             PORTD
                MOV             #$FF,PTAPUE  ;PORT A PULLUP ENABLE REGISTER
                MOV             #$6F,PTCPUE  ;PORT C PULLUP ENABLE REGISTER
                MOV             #$FF,PTDPUE  ;PORT D PULLUP ENABLE REGISTER
                MOV             #$FF,DDRB               ;PORT B SET TO OUTPUTS
                MOV             #$10,DDRC               ;PORT C BIT4 AS OUTPUT FOR LED
  CLR  Mode
  CLR  NextMode
                CLR             EventCount


* -------------------------------------------------------------------
* Main program code
* -------------------------------------------------------------------
 ;-----------------------------
 ;Port B setup for LCD module
 ;-----------------------------
  BCLR         RS,PORTB                ;Preset control line output levels
  BCLR         RW,PORTB
  BCLR         E_LCD,PORTB
  MOV  #$FE,DDRB  ;Setup PB1-PB7 as outputs for LCD
  BCLR         RS,PORTB  ;Preset control line output levels
  BCLR         RW,PORTB
  BCLR         E_LCD,PORTB

 ;-------------------------------------------------
 ;KBI setup
 ;-------------------------------------------------
  BSET            IMASKK,INTKBSCR         ;Mask out KBI
  BCLR            MODEK,INTKBSCR          ;edge-sensitive only

  MOV             #$F8,DDRA  ;Set buttons as inputs
  MOV  #$06,PTAPUE  ;Attach internal pullups for buttons
  MOV             #$06,INTKBIER           ;Turn on KBI 1 & 2

  BSET            ACKK,INTKBSCR           ;Clear pending interrupts

  BCLR         IMASKK,INTKBSCR  ;Enable keyboard interrupts


 ;-------------------------------------------------
 ;ADC setup
 ;-------------------------------------------------
 ;At 1.2288MHz, choose divisor of 1 (~1MHz)
  BSET         ADICLK,ADCLK         ;Select internal clock

 ;-------------------------------------------------
 ;Timer setup
 ;-------------------------------------------------
  BSET            TSTOP,T2SC              ;Stop the timer 2
  BSET            TRST,T2SC               ;Reset the timer 2
  LDA             T2SC
  ORA             #PRESCALER2             ;Set prescaler timer 2
  STA             T2SC

  BSET            TSTOP,T1SC              ;Stop the timer 1
  BSET            TRST,T1SC               ;Reset the timer 1
  LDA             T1SC
  ORA             #PRESCALER  ;Set prescaler timer 1
  STA             T1SC

  MOV             #TOFMAXCOUNT,TOFCount
  BSET            TOIE,T1SC  ;enable interupts
  BCLR         TSTOP,T1SC  ;start timer
  CLR             TempVar   ;initialize TempVar


  CLI     ;clear interupt mask

 ;-----------------------------
 ;LCD setup
 ;-----------------------------
  JSR             LCDInit   ;Initialize LCD

* -------------------------------------------------------------------
* Jump table interpreter
* -------------------------------------------------------------------
ModeJump:
  CLRH               ;Clear high index byte
  LDX             NextMode         ;Load X with mode value
  STX  Mode
  LSLX             ;Multiply by 4 for offset
  LSLX
  CLI     ;Globally enable interrupts
                BCLR            P4,PORTC                ;Turn off LED
  JMP             JumpTable,X      ;Jump to appropriate routine

* -------------------------------------------------------------------
* Jump table to switch between test modes
* NOP provides offset divisible by 2
* -------------------------------------------------------------------
JumpTable:
  JMP             InfoDemo
  NOP
  JMP  TempDemo
  NOP
  JMP  SongDemo
  NOP

* -------------------------------------------------------------------
* Demo program 1
* Start listing information strings
* -------------------------------------------------------------------
InfoDemo:
  LDHX            #FeatureList            ;Load offset of desired string

MainLoop:
  JSR  ShowString
  LDA             #$FA                    ;Number of mS delay between strings
  JSR  msDelay                 ;F0 = 240mS X 4 times = 1S Aprox
                LDA  #$FA
  JSR  msDelay
                LDA  #$FA
  JSR  msDelay
                LDA  #$FA
  JSR  msDelay


NextMsg         LDA         Mode
  CMP  NextMode
  BNE  InfoEnd
  LDA  ,X
  CMP  #EOT
  BEQ  GotNext
  AIX  #1
  BRA  NextMsg
GotNext         AIX  #1
  LDA  ,X
  CMP  #TABLEEND
  BEQ  InfoDemo
  BRA  MainLoop

InfoEnd         JMP             ModeJump

;    "1234567890123456" Max Characters
FeatureList:

  FCB  "     AVNET"
  FCB  EOT
  FCB  "DESIGN SERVICES"
  FCB  EOT
  FCB  " HELLO CUSTOMER"
  FCB  EOT
  FCB  "68HC908GP32 MCU"
  FCB  EOT
  FCB  "32k FLASH memory"
  FCB  EOT
  FCB  "512 bytes RAM"
  FCB  EOT
  FCB  "8MHz internal"
  FCB  EOT
  FCB  "4 timer channels"
  FCB  EOT
  FCB  "Input capture"
  FCB  EOT
  FCB  "Output compare"
  FCB  EOT
  FCB  "Buffered PWM"
  FCB  EOT
  FCB  "8 A/D channels"
  FCB  EOT
  FCB  "PLL for 32kHz"
  FCB  EOT
  FCB  "SCI and SPI"
  FCB  EOT
  FCB  TABLEEND


* -------------------------------------------------------------------
* Demo program 2
* Read the voltage from the temp sensor connected to
* ADC channel 0 and display the results on the LCD
*
* -------------------------------------------------------------------
TempDemo:
  JSR  ConvertInit
TestLoop:
  LDA  Mode
  CMP  NextMode
  BNE  TempEnd
  JSR  Convert
  LDA  #$64
  JSR  msDelay
  BRA  TestLoop
TempEnd:
  LDA  ADSCR
  ORA             #$1F                    ;Turn ADC off
  JMP  ModeJump

ConvertInit:
  LDHX            #CONVMSG  ;Send text string to LCD
  JSR  ShowString


* -------------------------------------------------------------------
* Meassure temperature using an ADC channel, and display the results
* Resolution of ADC is assumed to be 5/255 = .019608V
* The Sensor selected for this project(Analog Devices, TMP37)
* has a linear output factor of 20mV/degC. Therefore, with a 5V reference
* 1 bit is approximately equal to 1 degC.
* Better resolution could be attained by using a lower reference,
* but this is accurate enough for the demonstration
*
* -------------------------------------------------------------------
Convert:
  CLR  ADSCR   ;Select channel AD0 for read
  BRCLR           COCO,ADSCR,*         ;Wait for conversion to complete

  LDHX            #CONVMSG                ;Send text string to LCD
  JSR  ShowString

  LDA  #$C0   ;Set address for line 2
  JSR  LCDCtrl   ;(second half of first line)
  CLRH
  LDX             ADR   ;Get conversion result
  LSLX     ;Multipy by 2 for indexing into table

  LDA  TempTBL,X  ;Load ASCII value for tens digit
  JSR  LCDData   ;Show it
  INCX     ;step to ones digit
  LDA  TempTBL,X  ;Load ASCII value for ones digit
  JSR  LCDData   ;Show it

  LDA  #$DF   ;Deg Symbol
  JSR  LCDData
  LDA  #$20   ;Space
  JSR  LCDData
  LDA  #$43   ;"C" for Celsius
  JSR  LCDData

  RTS

* -----------------------------------------------------------------
*
*  Temerature to ASCII look up table
*
* -----------------------------------------------------------------

TempTBL         FCB  $30,$30
  FCB  $30,$31
  FCB  $30,$32
  FCB  $30,$33
  FCB  $30,$34
  FCB  $30,$35
  FCB  $30,$36
  FCB  $30,$37
  FCB  $30,$38
  FCB  $30,$39
  FCB  $31,$30
  FCB  $31,$31
  FCB  $31,$32
  FCB  $31,$33
  FCB  $31,$34
  FCB  $31,$35
  FCB  $31,$36
  FCB  $31,$37
  FCB  $31,$38
  FCB  $31,$39
  FCB  $32,$30
  FCB  $32,$31
  FCB  $32,$32
  FCB  $32,$33
  FCB  $32,$34
  FCB  $32,$35
  FCB  $32,$36
  FCB  $32,$37
  FCB  $32,$38
  FCB  $32,$39
  FCB  $33,$30
  FCB  $33,$31
  FCB  $33,$32
  FCB  $33,$33
  FCB  $33,$34
  FCB  $33,$35
  FCB  $33,$36
  FCB  $33,$37
  FCB  $33,$38
  FCB  $33,$39
  FCB  $34,$30
  FCB  $34,$31
  FCB  $34,$32
  FCB  $34,$33
  FCB  $34,$34
  FCB  $34,$35
  FCB  $34,$36
  FCB  $34,$37
  FCB  $34,$38
  FCB  $34,$39
  FCB  $35,$30
  FCB  $35,$31
  FCB  $35,$32
  FCB  $35,$33
  FCB  $35,$34
  FCB  $35,$35
  FCB  $35,$36
  FCB  $35,$37
  FCB  $35,$38
  FCB  $35,$39
  FCB  $36,$30
  FCB  $36,$31
  FCB  $36,$32
  FCB  $36,$33
  FCB  $36,$34
  FCB  $36,$35
  FCB  $36,$36
  FCB  $36,$37
  FCB  $36,$38
  FCB  $36,$39
  FCB  $37,$30
  FCB  $37,$31
  FCB  $37,$32
  FCB  $37,$33
  FCB  $37,$34
  FCB  $37,$35
  FCB  $37,$36
  FCB  $37,$37
  FCB  $37,$38
  FCB  $37,$39

CONVMSG         FCB  "Temp = "
  FCB  EOT


* -------------------------------------------------------------------
* Demo program 3
* Use the timer channel PWM generation to play a tune on a speaker
* -------------------------------------------------------------------
SongDemo:
                LDHX         #ToneString1
                JSR             ShowString
  LDHX         #TestTones
  STHX         TempPtr

NextLoop:
  LDA  Mode
  CMP  NextMode
  BNE  EndSong
  LDA  1,X
  STA  EventCount
  LDHX            TempPtr

  LDA  ,X
  AIX  #1
  STHX         TempPtr

  CMP  #TABLEEND
  BEQ  RepeatSong
  JSR  Tone

WaitLoop:
  LDA  EventCount
  BEQ  NextTone
  BRA  WaitLoop

NextTone:
  LDHX         TempPtr
  AIX  #1
  STHX         TempPtr
  BRA  NextLoop

RepeatSong:
  BRA  SongDemo

EndSong         JSR  NoSound
  JMP  ModeJump

;    "1234567890123456"   ;The length of the LCD line
ToneString1:
  FCB          "Using timer PWMs"
  FCB          EOT
;=======================================================
;"The Eyes of Texas"
;=======================================================
TestTones:
 FCB $01,$04,$04,$06,$01,$02,$04,$02,$01,02,$04,$02,$05,$02,$06,$08,$04,$07
 FCB $00,$01
 FCB $07,$04,$00,$02,$07,$02,$04,$04,$05,$04,$06,$08
 FCB $00,$04
 FCB $01,$04,$04,$06,$01,$02,$04,$02,$01,02,$04,$02,$05,$02,$06,$08,$04,$04
 FCB $00,$01
 FCB $06,$03,$00,$01,$06,$04,$05,$02,$00,$01,$05,$04,$06,$04,$05,$08
 FCB $00,$04
 FCB $05,$05,$00,$01,$05,$01,$04,$03,$05,$01,$06,$03,$05,$01,$04,$07,$01,$07
 FCB $00,$02
 FCB $07,$05,$00,$01,$07,$01,$04,$02,$00,$01,$04,$01,$05,$02,$00,$01,$05,$01,$06,$07
 FCB $00,$04
 FCB $01,$02,$02,$06,$03,$02,$04,$02,$03,$02,$04,$02,$02,$02,$01,$06,$04,$04
 FCB $00,$02
 FCB $05,$04,$06,$04,$07,$04,$06,$04,$05,$04,$04,$08
 FCB $00,$02
 FCB TABLEEND

;=======================================================

* -------------------------------------------------------------------
* Timer 1 overflow interrupt, used as a time base for melody generation
* -------------------------------------------------------------------
T1OFInt:
  BCLR         TOF,T1SC
  DBNZ         TOFCount,T1NoEvent
  MOV  #TOFMAXCOUNT,TOFCount
  DBNZ         EventCount,T1NoEvent       ;EventCount = 0 indicates load next entry

T1NoEvent: RTI

* -------------------------------------------------------------------
* Lookup table to convert a relative tone to an offset for the timer channel
* -------------------------------------------------------------------
ToneTable:
  FDB  0
  FDB  391
  FDB  346
  FDB  310
  FDB  294
  FDB  255
  FDB  231
  FDB  218
  FDB  208
  FDB  196
  FDB  173
  FDB  155
  FDB  147
  FDB  127
  FDB  115
  FDB  104
  FDB  98
;These are old tones. Not needed
  FDB  0
  FDB  247
  FDB  233
  FDB  220
  FDB  208
  FDB  196
  FDB  185
  FDB  175
  FDB  165
  FDB  155
  FDB  147
  FDB  130
  FDB  131
  FDB  123
  FDB  116
  FDB  110
  FDB  104
  FDB  98
  FDB  92
  FDB  87
  FDB  82
  FDB  78
  FDB  73
  FDB  69
  FDB  65


* -------------------------------------------------------------------
* Sound a tone using the PWM channel through a speaker
* -------------------------------------------------------------------
Tone         MOV  #$1A,T2SC1              ;Setup channel for PWM generation

* -------------------------------------------------------------------
*Convert tone offset to address in H:X
* -------------------------------------------------------------------

ToneLoop1:
  CLRH
  TAX
  ASLX
  LDA  ToneTable,X
  STA  TempWord
  LDA  ToneTable+1,X
  STA  TempWord+1
  JSR  LoadPWM
  RTS


ToneOff:
  BSET            TSTOP,T2SC              ;
  CLR  T2SC1
  RTS
*-----------------------------------------------------------
NoSound:
  BSET            TSTOP,T2SC
  BSET         TRST,T2SC
Return  RTS



*-----------------------------------------------------------
* Load values for modulus (frequency) and pulse width from
* table offset value
*-----------------------------------------------------------
LoadPWM PSHX             ;Save registers
  PSHA
  BSET            TSTOP,T2SC              ;Stop the timer
  BSET         TRST,T2SC         ;Reset the timer
 ;-------------------------------------------------
  MOV  TempWord,T2MODH
  MOV  TempWord+1,T2MODL

   ;CLR       T2MODH   ;Store to modulus
   ;MOV       LastVal,T2MODL

 ;-------------------------------------------------
                MOV  T2MODH,T2CH1H
             MOV  T2MODL,T2CH1L
  ASR  T2CH1H    ;Pulse width = period/2
             BCLR         7,T2CH1H   ;Get rid of 7th bit rotated in
  ASR  T2CH1L
             BCLR         7,T2CH1L
  BCLR            TSTOP,T2SC              ;Restart timer counter
 ;-------------------------------------------------
LoadDone:
  PULA
  PULX
  RTS


* -------------------------------------------------------------------
* LCD INITIALIZATION
* Called on startup to initialize the LCD into 4-bit mode
* Don't use the LCDControl subroutine at first, because initially
* the LCD is in 8-bit interface mode.
* -------------------------------------------------------------------
LCDInit:
 ;------------------------------------------
 ;8-bit interface at first
 ;------------------------------------------
  LDA  #$0F   ;Wait 15ms
  JSR  msDelay

  LDA  #$30   ;Function set
  JSR  LCD8Ctrl

  LDA  #5   ;Wait 5ms
  JSR  msDelay

  LDA  #$30   ;Function set
  JSR  LCD8Ctrl

  LDA  #1   ;Wait 1ms
  JSR  msDelay

  LDA  #$30   ;Function set
  JSR  LCD8Ctrl

  LDA  #$20   ;Function set first set up 4-bit
  JSR  LCD8Ctrl

 ;------------------------------------------
 ;4-bit operation
 ;------------------------------------------
  LDA  #$28   ;Second 4-bit function set
  JSR  LCDCtrl   ;Also 2-line setup

  LDA  #$06   ;Increment, no shift
  JSR  LCDCtrl

  LDA  #$01   ;Clear display, cursor home
  JSR  LCDCtrl

  LDA  #$0C   ;Display on, cursor off, blink off
  JSR  LCDCtrl

  RTS


* -------------------------------------------------------------------
* If the LCD is in 8-bit mode, just send the upper data bits through
* the lower port C bits
* Acc contains control byte
* -------------------------------------------------------------------
LCD8Ctrl:
  AND  #$F0   ;Mask out lower data bits
  PSHA     ;Store value to send temporarily
  LDA  PORTB   ;Don't change other PORTB pins
  AND  #$0F   ;Mask out UPPER nibble
  ADD  1,SP   ;Add value to send
  STA  PORTB   ;Store to PORTB

  BSET         E_LCD,PORTB   ;Toggle enable line
  BCLR         E_LCD,PORTB


  JSR  _40usDelay  ;40us setup time

  PULA     ;Deallocate stack data
  RTS

* -------------------------------------------------------------------
* Write LCD control byte in 4-bit mode
* For 4-bit mode, need to send the nibbles one at a time, high nibble first
* Acc contains control byte to send
* This routine is also used for Address writes to LCD, when MSbit of
* data byte is set. (i.e. $80 for address 00)
* -------------------------------------------------------------------
LCDCtrl:
 ;------------------------------------------
 ;Upper nibble
 ;------------------------------------------
  PSHA     ;Store data on stack
  AND  #$F0   ;Mask out lower nibble
  PSHA     ;Store upper nibble on stack
  LDA  PORTB   ;Load PORTB contents
  AND  #$0F   ;Mask out UPPER nibble
  ADD  1,SP   ;Add the data nibble
  STA  PORTB   ;Present upper nibble to LCD

  BSET         E_LCD,PORTB   ;Toggle Enable line
  BCLR         E_LCD,PORTB


  JSR         _40usDelay  ;40us setup time

 ;------------------------------------------
 ;Lower nibble
 ;------------------------------------------
  PULA     ;Deallocate last temp storage
  PULA     ;Get original data byte
  AND  #$0F   ;Mask out upper nibble
  NSA     ;Put LOWER nibble in UPPER nibble
  PSHA     ;Store onto stack
  LDA  PORTB   ;Get existing PORTB data
  AND  #$0F   ;Mask out lower nibble
  ADD  1,SP   ;Add lower nibble of data byte
  STA  PORTB   ;Store to PORTB

  BSET         E_LCD,PORTB   ;Toggle Enable line
  BCLR         E_LCD,PORTB

  PULA     ;Deallocate temp storage
  CMP  #$10   ;Longer delay for commands 1 or 2
  BEQ  LCLonger
  CMP  #$20
  BEQ  LCLonger

  JSR  _40usDelay  ;40us for any other command
  RTS     ;Return

LCLonger:
  LDA  #2
  JSR  msDelay
  RTS

* -------------------------------------------------------------------
* Write data byte to LCD, using 4-bit mode
* Acc contains data byte to send
* -------------------------------------------------------------------
LCDData:
 ;------------------------------------------
 ;First nibble
 ;------------------------------------------
  PSHA     ;Store data on stack temporarily
  AND  #$F0   ;Mask out lower nibble
  PSHA     ;Store upper nibble on stack
  LDA  PORTB   ;Load PORTB contents
  AND  #$0F   ;Mask out UPPER nibble
  ADD  1,SP   ;Add the data nibble

  STA  PORTB   ;Store to PORTB

  BSET         RS,PORTB  ;Set RS for control

  BSET         E_LCD,PORTB   ;Toggle Enable line
  BCLR         E_LCD,PORTB


 ;------------------------------------------
 ;Second nibble
 ;------------------------------------------
  PULA     ;Deallocate temp storage
  PULA     ;Get original data
  AND  #$0F   ;Mask out upper nibble
  NSA     ;Put LOWER nibble in UPPER port pins
  PSHA     ;Store onto stack
  LDA  PORTB
  AND  #$0F
  ADD  1,SP
  STA  PORTB

  BSET         E_LCD,PORTB   ;Toggle enable line
  BCLR         E_LCD,PORTB


  JSR  _40usDelay  ;40us setup time
  JSR  _40usDelay  ;40us setup time

  BCLR         RS,PORTB  ;Clear RS for data

  JSR  _40usDelay  ;40us setup time
  JSR  _40usDelay  ;40us setup time


  PULA     ;Deallocate temp storage
  RTS

* -------------------------------------------------------------------
* Clear the LCD display by sending the appropriate command byte
* -------------------------------------------------------------------
LCDClear:
  LDA  #$01   ;Clear display AND home cursor
  JSR  LCDCtrl
  RTS

LCDHome         LDA  #$02
  JSR  LCDCtrl
  RTS

* -------------------------------------------------------------------
* DELAY ROUTINES
* -------------------------------------------------------------------
* 1ms delay loop, causes _roughly_ 1.3ms delay @ fop = 8MHz
* uses constant for loop control
* cycles = 4 + X(6+7+1275) + 3 + 6
* cycles = 13 + X(1288)
* where X is value loaded into Acc
* Causes 1.3ms delay for BUSFREQ values of (1-8 integer values)
* -------------------------------------------------------------------
_1msDelay:
  PSHA     ;2 cycles
  LDA  #BUSFREQ  ;2
DLLoop         DBNZA         DLSub   ;3
  BRA  DLDone   ;3
DLSub         MOV  #$FF,Counter          ;4
  DBNZ         Counter,*  ;5
  BRA  DLLoop   ;3
DLDone         PULA     ;2
  RTS     ;4

* -------------------------------------------------------------------
* Variable ms delay loop
* Calls DelayLoop Acc number of times
* -------------------------------------------------------------------
msDelay         JSR            _1msDelay
  DBNZA           msDelay
  RTS

* -------------------------------------------------------------------
* 40usec delay routine. Important for LCD module, which requires 40us
* delay for many of its commands, including the "data setup time"
* 6+X(3)
* Where X = BUSFREQ*USCOUNT = USDELAY
* Provides _roughly_ 40us delay
* -------------------------------------------------------------------
_40usDelay:
  LDA  #BUSFREQ*USCOUNT        ;2
  DBNZA         *   ;3
  RTS                                     ;4

* -------------------------------------------------------------------
* Big delay loop
* Calls msDelay a predefined number of times
* -------------------------------------------------------------------
BigDelay:
  PSHA
  LDA  #$0A
  JSR  msDelay
  PULA
  RTS
* -------------------------------------------------------------------
* Bigger delay loop
* Calls msDelay a predefined number of times
* -------------------------------------------------------------------
BiggerDelay:
  PSHA
  LDA  #$FF
  JSR  msDelay
  LDA  #$FF
  JSR  msDelay
  LDA  #$FF
  JSR  msDelay
  LDA  #$FF
  JSR  msDelay
  PULA
  RTS

* -------------------------------------------------------------------
* Vector trap to detect spurious exceptions. (no RTI)
* -------------------------------------------------------------------
Trap           LDHX         #ErrorMsg
  JSR         ShowString
  BRA  *

ErrorMsg:
  FCB  "Vector trapped"
  FCB         EOT

*-------------------------------------------------------------
* Scroll subroutines
*-------------------------------------------------------------
* Initialize the message variables for the desired output string
* Register A contains the offset of desired message.
*-------------------------------------------------------------
LoadMsg         STHX            MsgIndex  ;Setup the message index
  STHX            MsgStart    ;Store the start of the message
  RTS                        ;Return

*-------------------------------------------------------------
* Update the LCD with current portion of string to be displayed
*-------------------------------------------------------------
UpdateLCD:
  LDHX            MsgIndex                ;Start at current index into message
  JSR            ShowString  ;Show current portion of string
  LDHX            MsgIndex
  CPHX         MsgStart
  BNE  ULgo
  LDA  #$30
  JSR  msDelay
ULgo         AIX  #1
  STHX            MsgIndex    ;Increment the index
  RTS                        ;Return

*-------------------------------------------------------------
* Show the current string portion on the display.
* When called, the X register contains the index offset.
*-------------------------------------------------------------
ShowString:
  JSR             LCDClear
  CLR            Count       ;Clear the counter variable
  LDA  #$80   ;Starting address
  JSR  LCDCtrl
NextByte:
  LDA  ,X        ;Load ASCII byte of string
  CMP            #EOT        ;Check for end of string
  BEQ            Padding     ;Last character reached
  JSR  LCDData
  AIX            #1        ;Increment the index
  INC            Count       ;Increment the counter
  LDA            Count       ;Check the counter
  CMP            #MAXCHARS   ;for LCD display length
  BEQ            Done        ;End of display line reached
  CMP  #MAXLINE
  BNE  SSCont
  LDA  #$C0   ;Set address for line 2
  JSR  LCDCtrl
SSCont         BRA            NextByte    ;Ready the next byte
Padding         LDA            Count       ;Pad the rest of the display with spaces
  CMP            #$00        ;See if string has scrolled off display
  BEQ            Reset       ;Need to reset string
  CMP            #MAXCHARS   ;Check for end of display
  BEQ            Done        ;Finished displaying padding spaces
  INC            Count       ;Increment counter
  JSR            BlankSpace  ;Put space in current display position
  BRA            Padding      ;Repeat
Reset         JSR            BlankSpace   ;Show a final space in first position
  LDHX            MsgStart     ;Load start of message index
  AIX  #-1                     ;Compensate for INCX in UpdateLCD after RTS
  STHX            MsgIndex    ;Record new message index
Done         RTS                        ;Return

* -------------------------------------------------------------------
* Sends an ASCII space character to the LCD
* -------------------------------------------------------------------
BlankSpace:
  LDA  #$20
  JSR  LCDData
  RTS

* -------------------------------------------------------------------
* Keyboard interrupt routine
* -------------------------------------------------------------------
KBInt:          BSET            P4,PORTC                ;Turn on LED
  JSR  BigDelay  ;Debounce a bit
  LDA             PORTA   ;Read DIP
  BSET            ACKK,INTKBSCR         ;Clear pending interrupts

* -------------------------------------------------------------------
* Button test
* Poll (PA1 & PA2)
* -------------------------------------------------------------------

KBLoop:
  AND             #$06   ;Mask unused bits
  CMP  #$06
  BEQ  KBDone   ;neither button pushed, do nothing
  CMP  #$00
  BEQ  KBDone   ;Both buttons pushed, do nothing
  CMP  #$02
  BEQ  SetNext   ;SW2 pressed do next demo
  CMP  #$04
  BEQ  SetPrev   ;SW1 pressed do previous demo

SetNext:
  LDA  Mode
  INCA        ;Increment Next Demo to next demo
  CMP  #LastMode  ;Compare to last
  BLS  NotDone   ;If less then or equal to last demo, do it
  CLRA     ;If greater then last demo, start over
NotDone:
  STA  NextMode  ;store value for next demo
  BRA  KBDone

SetPrev:
  DEC  NextMode
  BPL  KBDone
  MOV  #LastMode,NextMode
  BRA  KBDone

KBDone RTI

* -------------------------------------------------------------------
* VECTOR ASSIGNMENTS
* -------------------------------------------------------------------
* Trap unused vectors to indicate errors
* -------------------------------------------------------------------
         ORG            TBVEC
         FDB            Trap
         ORG            ADCVEC
         FDB            Trap
         ORG            KBIVEC
         FDB            KBInt
         ORG            SCITXVEC
         FDB            Trap
         ORG            SCIRXVEC
         FDB            Trap
         ORG            SCIERVEC
         FDB            Trap
         ORG            SPITXVEC
         FDB            Trap
         ORG            SPIRXVEC
         FDB            Trap
         ORG            T2OFVEC
         FDB            Trap
         ORG            T2CH1VEC
         FDB            Trap
         ORG            T2CH0VEC
         FDB            Trap
         ORG            T1OFVEC
         FDB            T1OFInt
         ORG            T1CH1VEC
         FDB            Trap
         ORG            T1CH0VEC
         FDB            Trap
         ORG            PLLVEC
         FDB            Trap
         ORG            IRQ1VEC
         FDB            Trap
         ORG            SWIVEC
         FDB            Trap
         ORG            RESETVEC
         FDB            Start
